with Unchecked_Deallocation;

package body Listas_Recursivas is
	-- Procedimiento para liberar nodos de la lista
	procedure Liberar is new Unchecked_Deallocation(TNodo,Lista);	

	function Crear(E: in TElemento; L: in Lista) return Lista is
	begin
		-- Se crea un nodo poniendo E como informacin y enganchando
		-- L como siguiente
		return new TNodo'(E,L);
	exception
		-- La excepcin Storage_Error ocurre cuando no hay espacio para
		-- crear un nodo. Se cambia por E_No_Espacio
		when Storage_Error => raise E_No_Espacio;
  	end Crear;

	function Primero(L: in Lista) return TElemento is
	begin
		-- Se devuelve el campo de informacin del primer nodo
		return L.Info;
	exception
		-- Si L est vaca, ocurre una excepcin Constraint_Error al
		-- intentar acceder al campo Info. Se cambia por E_Lista_Vaca
		when Constraint_Error => raise E_Lista_Vaca;
	end Primero;

	function Resto(L: in Lista) return Lista is
	begin
		-- Se devuelve la direcin almacenada en el campo siguiente del
		-- primer nodo
		return L.Sig;
	exception
		-- Si L est vaca, ocurre una excepcin Constraint_Error al
		-- intentar acceder al campo Sig. Se cambia por E_Lista_Vaca
		when Constraint_Error => raise E_Lista_Vaca;
	end Resto;

   function Es_Vaca(L: in Lista) return boolean is
   begin
		-- La lista vaca se representa por el puntero nulo
		return L = null;
	end Es_Vaca;

	function Tamao(L: in Lista) return integer is
	begin
		-- El tamao se calcula recorriendo la lista recursivamente
		if Es_Vaca(L) then
			return 0;
		else
			return 1 + Tamao(Resto(L));
		end if;
	end Tamao;

	function Copia(L: in Lista) return Lista is
		-- Devuelve una copia de L
	begin
		-- La lista se copia recursivamente
		if Es_Vaca(L) then
			return null;
		else
			return new TNodo'(Primero(L),Copia(Resto(L)));
		end if;
	end Copia;

	procedure Asignar(Destino: in out Lista; Fuente: in Lista) is
		Aux: Lista;
	begin
		-- Si el destino est vaco no hacen falta otras comprobaciones
		if Destino /= null then
			-- Est el destino contenido en la fuente?	
			Aux := Fuente;
			while (aux /= null) and then (Aux /= Destino) loop
				Aux := Aux.Sig;
			end loop;

			-- Si Aux es null, el destino no est contenido en la fuente
			if Aux = null then
				-- Est la fuente contenida en el destino?
				Aux := Destino;
				while (Aux /= null) and then (Aux /= Fuente) loop
					Destino := Destino.Sig;
					Liberar(Aux);
					Aux := Destino;
				end loop;

				-- Si Aux no es nulo, lo que queda del destino coincide
				-- con la fuente
				if Aux = null then
					Destino := Copia(Fuente);
				end if;
			else -- El destino est contenido en la fuente
				Destino := Fuente;
			end if;
		else
			Destino := Copia(Fuente);
		end if;
	end Asignar;
end Listas_Recursivas;
