generic
    type TElemento is private;
package Listas_Recursivas is
    type Lista is limited private;
    
    Vaca : constant Lista;
	-- Representa una lista vaca

    function Crear(E: in TElemento; L: in Lista) return Lista;
	-- Crea una lista uniendo un elemento, E, con una lista ya existente, L.
	-- E se convierte en primero de la nueva lista, L se emplea con 
	-- semntica de alias.
	-- Si no puede aadir el elemento lanza la excepcin E_No_Espacio.

    function Primero(L: in Lista) return TElemento;
	-- Devuelve el primer elemento de una lista no vaca.
	-- Si L est vaca lanza la excepcin E_Lista_Vaca;

    function Resto(L: in Lista) return Lista;
	-- Si L no est vaca, devuelve, con semntica de alias la sublista
	-- de L formada por todos los elementos menos el primero.
	-- Si L est vaca lanza la excepcin E_Lista_Vaca;

    function Es_Vaca(L: in Lista) return boolean;
	-- Evala si L es la lista vaca.

	function Tamao(L: in Lista) return integer;
	-- Devuelve el nmero de elementos de L.

	procedure Asignar(Destino: in out Lista; Fuente: in Lista);
	-- Asigna Fuente a Destino con semntica de copia.

	-- Se lanza cuando, en una lista vaca, se intenta realizar
	-- una operacin no aplicable a las listas vacas.
	E_Lista_Vaca: exception;

	-- Se lanza cuando no se puede crear un nodo por falta de espacio.
	E_No_Espacio : exception;
private
	-- La estructura de Lista es la de una lista simplemente encadenada
    type TNodo is record
        Info: TElemento;
        Sig : Lista;
    end record;
            
    type Lista is access TNodo;

    Vaca : constant Lista := null;
end Listas_Recursivas;
