with Listas_Recursivas;

generic
	type TElemento is private;
package Listas_Secuenciales is
	type Lista is limited private;
	-- Cuando se crea una variable de tipo Lista, est vaca

	procedure Insertar(L: in out Lista; E: TElemento);
	-- Aade un elemento en la posicin actual, que pasa a
	-- ocupar dicha posicin, si queda espacio.
	-- Si no queda espacio se lanza la excepcin E_Lista_Llena

	procedure Extraer(L: in out Lista);
	-- Elimina el elemento que ocupa la posicin actual. El siguiente
	-- elemento pasa a ocupar dicha posicin, si existe, o queda
	-- sealando al final si se extrae el ltimo.
	-- Cuando la posicin actual se encuentra al final de la lista
	-- se lanza la excepcin E_Fin_Lista.

	procedure Avanzar(L: in out Lista);
	-- Modifica la posicin actual haciendo que seale al siguiente
	-- elemento en la lista, si existe, o queda sealando al final
	-- si se avanza el ltimo.
	-- Cuando la posicin actual se encuentra al final de la lista
	-- se lanza la excepcin E_Fin_Lista.

	procedure Retroceder(L: in out Lista);
	-- Modifica la posicin actual haciendo que seale al elemento
	-- que le precede en la lista, si existe.
	-- Cuando la posicin actual se encuentra al principio de la lista
	-- se lanza la excepcin E_Inicio_Lista.

	procedure Inicio(L: in out Lista);
	-- Modifica la posicin actual haciendo que seale al principio
	-- de la lista, dando acceso al primer elemento si la lista no
	-- est vaca.

	procedure Fin(L: in out Lista);
	-- Modifica la posicin actual haciendo que seale al final
	-- de la lista, detrs del ltimo elemento.

	function Es_Inicio(L: in Lista) return Boolean;
	-- Evala si la posicin actual se encuentra al principio de la lista.

	function Es_Fin(L: in Lista) return Boolean;
	-- Evala si la posicin actual se encuentra al final de la lista.

	function Examinar(L: in Lista) return TElemento;
	-- Devuelve el elemento actual de L

	function Es_Vaca(L: in Lista) return Boolean;
	-- Evala si L est vaca

	function Tamao(L: in Lista) return Natural;
	-- Devuelve el nmero de elementos almacenados en L

	-- Se lanza cuando no se puede insertar un elemento 
	-- por falta de espacio.
	E_Lista_Llena : exception;

	-- Se lanza cuando se intenta realizar una operacin no aplicable
	-- cuando la posicin actual se encuentra al principio de la lista.
	E_Inicio_Lista: exception;

	-- Se lanza cuando se intenta realizar una operacin no aplicable
	-- cuando la posicin actual se encuentra al final de la lista.
	E_Fin_Lista: exception;

private
	package Lista_Elementos is new Listas_Recursivas(TElemento);
	use Lista_Elementos;

	type Lista is record
		-- sublista de los elementos anteriores al actual
		Anteriores : Lista_Elementos.Lista;
		-- sublista del elemento actual y los posteriores
		Posteriores: Lista_Elementos.Lista;
		Tamao     : natural := 0;
	end record;
end Listas_Secuenciales;
