#ifndef INC_LISTA_RECURSIVA
#define INC_LISTA_RECURSIVA
#include <cstdlib>
template <class T>
class ListaRecursiva {
   public:
   // Constructoras
      /**
       * Representa la lista vaca
       */
      ListaRecursiva();
   
      /**
       * Crea una lista aadiendo al principio de una lista ya existente 
       * un nuevo elemento con semntica de alias
       */
      ListaRecursiva(const T &,const ListaRecursiva<T> &);
   
      /**
       * Crea una lista eliminando de una lista ya existente el primer
       * elemento con semntica de alias, si no est vaca
       */
      ListaRecursiva<T> resto() const; 
   
      /**
       * Sobrecarga del operador de asignacin: se garantiza que la lista
       * destino es una copia de la que se pasa por parmetro, aunque en
       * pos de la eficiencia, si la lista destino coincide con la fuente,
       * total o parcialmente, es inncesario copiarla toda
       */
      ListaRecursiva<T> & operator =(const ListaRecursiva<T> &);
   
   // Observadoras
      /**
       * Devuelve el primer elemento de una lista no vaca
       */
      T primero() const;
   
      /**
       * Devuelve verdadero si el nmero de elementos es distinto de cero
       */
      bool hayElementos() const;
   
      /**
       * Devuelve el nmero de elementos de la lista
       */
      int cardinal() const;
   private:
      /**
       * El tipo Nodo es privado, ya que su razn de ser es servir de 
       * "ladrillo" para formar la estructura de la Lista como 
       * lista simplemente encadenada
       */
      struct Nodo {
         T elemento;
         ListaRecursiva<T> siguiente;
         Nodo(T e, ListaRecursiva<T> s):elemento(e),siguiente(s){}
         ~Nodo(){}
      };
      typedef Nodo *PNodo;
      PNodo lista;  // Puntero al primer nodo de la lista encadenada
   
      /**
       * Realiza una copia de una lista simplemente encadenada
       */
      ListaRecursiva<T> copia() const;
};

/**
 * Realiza una copia de una lista simplemente encadenada
 */
template<class T>
ListaRecursiva<T> ListaRecursiva<T>::copia() const{         
   if (!hayElementos()) {
      return ListaRecursiva<T>();
   }
   else {
      return ListaRecursiva<T>(primero(),lista->siguiente.copia());
   }
};

/**
 * Representa la Lista vaca
 */
template <class T>
ListaRecursiva<T>::ListaRecursiva() {
   lista = NULL;
};

/**
 * Crea una lista aadiendo al principio de una lista ya existente
 * un nuevo elemento con semntica de alias
 */
template <class T>
ListaRecursiva<T>::ListaRecursiva(const T &e,const ListaRecursiva<T> &l) {
   lista = new Nodo(e,l);
};

/**
 * Crea una lista eliminando de una lista ya existente el primer
 * elemento con semntica de alias, si no est vaca 
 */
template <class T>
ListaRecursiva<T> ListaRecursiva<T>::resto() const { 
   return lista->siguiente;
}; 

/**
 * Sobrecarga del operador de asignacin: se garantiza que la lista
 * destino es una copia de la que se pasa por parmetro, aunque en
 * pos de la eficiencia, si la lista destino coincide con la fuente,
 * total o parcialmente, es inncesario copiarla toda
 */
template <class T>
ListaRecursiva<T> & ListaRecursiva<T>::operator=
		(const ListaRecursiva<T> &l) {
      // Si el destino est vaco no hacen falta otras comprobaciones
   if (lista != NULL) {
      // Est el destino contenido en la fuente?
      PNodo pAux = l.lista;
      while (pAux != NULL) {
         if (pAux == lista) 
            break;
         pAux = pAux->siguiente.lista;
      }
      // Si pAux es NULL, el destino no est contenido en la fuente
      if (pAux == NULL) {
         // Est la fuente contenida en el destino?
         pAux = lista;
         while (pAux != NULL) {
            if (pAux == l.lista) 
               break;
            lista = lista->siguiente.lista;
            delete pAux;
            pAux = lista;
         }
         // Si pAux no es NULL, lo que quedaba del destino coincide 
         // con la fuente
         if (pAux == NULL) lista = l.copia().lista;
      }
      else // El destino est contenido en la fuente
         lista = l.lista;
   } 
   else
      lista = l.copia().lista;
   return *this;
};

/**
 * Devuelve el primer elemento de una lista no vaca
 */
template <class T>
T ListaRecursiva<T>::primero() const {
   return lista->elemento;
};

/**
 * Devuelve verdadero si el nmero de elementos es distinto de cero
 */
template <class T>
bool ListaRecursiva<T>::hayElementos() const {
   return lista != NULL;
};

/**
 * Devuelve el nmero de elementos de la lista
 */
template <class T>
int ListaRecursiva<T>::cardinal() const {
   if (hayElementos())
      return 1 + resto().cardinal();
   else
      return 0;
};

#endif
