#ifndef INC_PILA
#define INC_PILA
#include <exception>
#include "listarecursiva.h"
   using namespace std;
template <class T>
class Pila {
   public:
   // Constructoras
      /**
       * Representa la pila vaca
       */
      Pila();
   
      /**
       * Destructor
       */
      ~Pila();
   
      /**
       * Aade un elemento en la cima de la pila
       */
      void apilar(const T &); 
   
      /**
       * Quita el elemento que ocupa la cima de la pila.
       * Si la pila est vaca lanza la excepcin NoHayElementos
       */
      void desapilar(); 
   
   // Observadoras
      /**
       * Devuelve el elemento que ocupa la cima de la pila.
       * Si la pila est vaca lanza la excepcin NoHayElementos
       */
      T examinar() const;
   
      /**
       * Devuelve verdadero si el nmero de elementos es distinto de cero
       */
      bool hayElementos() const;
   
      /**
       * Devuelve el nmero de elementos de la pila
       */
      int cardinal() const;      
      class NoHayElementos: public exception{};
   private:
      ListaRecursiva<T> elementos; 
      int numElementos;
      /**
       * Al defifir la sobrecarga del operador de asignacin
       * en la parte private se consigue que no se permita la 
       * asignacin entre pilas en el uso de la clase Pila
       */
      Pila<T> & operator =(const Pila<T> &){};
};

/**
 * Representa la pila vaca
 */
template <class T>
Pila<T>::Pila() {
   elementos = ListaRecursiva<T>();
   numElementos = 0;
};

/**
 * Destructor
 */
template <class T>
Pila<T>::~Pila() {
   elementos = ListaRecursiva<T>();
};

/**
 * Aade un elemento en la cima de la pila
 */
template <class T>
void Pila<T>::apilar(const T &e) {
   elementos = ListaRecursiva<T>(e,elementos);
   numElementos++;
};

/**
 * Quita el elemento que ocupa la cima de la pila.
 * Si la pila est vaca lanza la excepcin NoHayElementos
 */
template <class T>
void Pila<T>::desapilar() { 
   if (cardinal() == 0) throw NoHayElementos();
  
   elementos = elementos.resto();
   numElementos--;
}; 

/**
 * Devuelve el elemento que ocupa la cima de la pila.
 * Si la pila est vaca lanza la excepcin NoHayElementos
 */
template <class T>
T Pila<T>::examinar() const {
   if (cardinal() == 0) throw NoHayElementos();
 
   return elementos.primero();
};

/**
 * Devuelve verdadero si el nmero de elementos es distinto de cero
 */
template <class T>
bool Pila<T>::hayElementos() const {
   return numElementos != 0;
};

/**
 * Devuelve el nmero de elementos de la pila
 */
template <class T>
int Pila<T>::cardinal() const {
   return numElementos;
};

#endif
