package listasSecuenciales;
import copiable.*;
import listasRecursivas.*;

/**
 * Implementacin de una lista secuencial
 */
public class ListaSecuencial {
	/**
	 * Se lanza cuando se intenta realizar una operacin no aplicable
	 * cuando la posicin actual se encuentra al final de la lista
	 */
	public class ErrorFinLista extends RuntimeException {
	}

	/**
	 * Se lanza cuando se intenta realizar una operacin no aplicable
	 * cuando la posicin actual se encuentra al principio de la lista
	 */
	public class ErrorInicioLista extends RuntimeException {
	}

	/*
	 * La estructura de la lista consta de dos sublistas, una con los
	 * elementos anteriores al actual y otra con el actual y los posteriores
	 */
	ListaRecursiva anteriores, posteriores;
	int numElementos;
	
	/**
	 * Inicializa una lista vaca
	 */
	public ListaSecuencial() {
		anteriores  = new ListaRecursiva();
		posteriores = new ListaRecursiva();
		numElementos= 0;
	}

	/**
	 * Inserta e en la posicin actual
	 */
	public void insertar(Cloneable2 e) {
		posteriores = new ListaRecursiva(e, posteriores);
		numElementos++;
	}
	
	/**
	 * Elimina el elemento que ocupa la posicin actual. El siguiente
	 * elemento pasa a ocupar dicha posicin, si existe, o la posicin
	 * actual queda ms all del final, si se extrae el ltimo.
	 * Cuando, antes de extraer, la posicin actual se encuentra ms all 
	 * del final de la lista, se lanza la excepcin ErrorFinLista
	 */
	public void extraer() {
		if (esFin()) throw new ErrorFinLista();
		
		posteriores = posteriores.resto();
		numElementos--;
	}
	
	/**
	 * @return true si la posicin actual se encuentra ms all 
	 * del final de la lista
	 */
	public boolean esFin() {
		return posteriores.esVaca();
	}
	
	/**
	 * @return true si la posicin actual se encuentra al principio de la
	 * lista
	 */
	public boolean esInicio() {
		return anteriores.esVaca();
	}
	
	/**
	 * Modifica la posicin actual haciendo que seale al siguiente
	 * elemento en la lista, si existe o, si no, la posicin
	 * actual queda ms all del final.
	 * Cuando, antes de avanzar, la posicin actual se encuentra ms all 
	 * del final de la lista, se lanza la excepcin ErrorFinLista
	 */
	public void avanzar() {
		if (esFin()) throw new ErrorFinLista();

		anteriores   = new ListaRecursiva(posteriores.primero(), anteriores);
		posteriores  = posteriores.resto();
	}
	
	/**
	 * Modifica la posicin actual haciendo que seale al elemento
	 * que le precede en la lista, si existe.
	 * Cuando la posicin actual se encuentra al principio de la lista
	 * se lanza la excepcin ErrorInicioLista
	 */
	public void retroceder() {
		if (esInicio()) throw new ErrorInicioLista();
		
		posteriores = new ListaRecursiva(anteriores.primero(), posteriores);
		anteriores  = anteriores.resto();
	}
	
	/**
	 * Modifica la posicin actual haciendo que seale al principio
	 * de la lista, dando acceso al primer elemento si la lista no
	 * est vaca
	 */
	public void Inicio() {
		while (!esInicio()) retroceder();
	}
	
	/**
	 * Modifica la posicin actual haciendo que seale al final
	 * de la lista, detrs del ltimo elemento
	 */
	public void Fin() {
		while (!esFin()) avanzar();
	}
	
	/**
	 * @return el elemento actual. Si la posicin actual se encuentra
	 * ms all del final de la lista, lanza la excepcin ErrorFinLista
	 */
	public Cloneable2 examinar() {
		if (esFin()) throw new ErrorFinLista();

		return posteriores.primero();
	}

	/**
	 * @return true si la lista est vaca
	 */
	public boolean esVaca() {
		return numElementos == 0;
	}

	/**
	 * @return el nmero de elementos de la lista
	 */
	public int tamao() {
		return numElementos;
	}
}
