package pilas;
import copiable.*;
import listasRecursivas.*;

/**
 * Implementacin de una pila
 */
public class Pila {
	/**
	 * Se lanza cuando se intenta realizar una operacin no aplicable
	 * cuando la pila est vaca
	 */
	public class PilaVaca extends RuntimeException {
	}
	
	/*
	 * La estructura de la pila se basa una lista recursiva
	 */
	ListaRecursiva elementos = new ListaRecursiva(); 
	int numElementos = 0;
	
	/**
	 * Inserta e en la cima de la pila
	 * @param e
	 */
	public void apilar(Cloneable2 e) {
		elementos = new ListaRecursiva(e,elementos);
		numElementos++;
	}
	
	/**
	 * Elimina el elemento de la cima de la pila. Si la pila est
	 * vaca, lanza la excepcin PilaVaca 
	 */
	public void desapilar() {
		if (tamao() == 0) throw new PilaVaca();
		 
		elementos = elementos.resto();
		numElementos--;
	}

	/**
	 * @return el elemento de la cima de la pila. Si la pila est
	 * vaca, lanza la excepcin PilaVaca
	 */
	public Cloneable2 examinar() {
		if (tamao() == 0) throw new PilaVaca();

		return elementos.primero();
	}
	
	/**
	 * @return true si la pila est vaca
	 */
	public boolean esVaca() {
		return elementos.esVaca();
	}
	
	/**
	 * @return el nmero de elementos de la pila
	 */
	public int tamao() {
		return numElementos;
	}
}
